"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@code-engine/utils");
const ono_1 = require("ono");
const clone_context_1 = require("../clone/clone-context");
const clone_file_1 = require("../clone/clone-file");
const messenger_1 = require("../worker-thread/messenger");
const import_module_1 = require("./import-module");
/**
 * Executes commands in a worker thread that are sent by a corresponding `Worker` running on the main thread.
 * @internal
 */
class Executor extends messenger_1.Messenger {
    constructor(threadId, port) {
        super(port);
        this._processors = new Map();
        this.threadId = threadId;
    }
    /**
     * Loads the specified JavaScript module.
     */
    async loadModule(message) {
        let { moduleUID, moduleId, cwd } = message;
        let fileProcessor;
        // Import the plugin module
        let exports = await import_module_1.importModule(moduleId, cwd);
        // Get the default export, which must be a function
        let fn = (exports && exports.default) || exports;
        if (typeof fn !== "function") {
            throw ono_1.ono.type({ workerId: this.threadId, moduleId }, `Error loading module "${moduleId}". CodeEngine plugin modules must export a function.`);
        }
        if (message.data === undefined) {
            // The exported function is the FileProcessor
            fileProcessor = fn;
        }
        else {
            // The exported function is a factory that produces the FileProcessor function.
            // Call the factory with the given data.
            fileProcessor = await fn(message.data);
            if (typeof fileProcessor !== "function") {
                throw ono_1.ono.type({ workerId: this.threadId, moduleId }, `Error loading module "${moduleId}". The ${fn.name || "exported"} function should return a CodeEngine file processor.`);
            }
        }
        this._processors.set(moduleUID, fileProcessor);
        // Reply with information about the module
        this.postReply({ to: message.id, type: "moduleLoaded", name: fileProcessor.name });
    }
    /**
     * Processes a file using the specified plugin.
     */
    async processFile(message) {
        // Create clones of the File and Context
        let file = utils_1.createFile(message.file);
        let context = clone_context_1.createContext(this, message.id, message.context);
        // Process the file using the specified plugin
        let fileProcessor = this._processors.get(message.moduleUID);
        let output = await fileProcessor.call(undefined, file, context);
        for await (let fileInfo of utils_1.iterate(output)) {
            let outFile = utils_1.createFile(fileInfo);
            let [outFileClone, transferList] = clone_file_1.cloneFile(outFile);
            this.postReply({ to: message.id, type: "file", file: outFileClone }, transferList);
        }
        // Let the worker know that we're done yielding files
        this.postReply({ to: message.id, type: "finished" });
    }
}
exports.Executor = Executor;
//# sourceMappingURL=executor.js.map